@@ -1,5 +1,7 @@
 """
 Libreria per fare grafici di fit e tabelle LaTeX a partire da dati raccolti in un file Excel
+
+TODO: aggiungere comandi extra prima di salvere il grafico (not working)
 """
 __docformat__="numpy"
 
@@ -32,24 +34,49 @@ def _tex_escape(text):
     regex = re.compile('|'.join(re.escape(str(key)) for key in sorted(conv.keys(), key = lambda item: - len(item))))
     return regex.sub(lambda match: conv[match.group()], text)
 
+def _converti_latex(text):
+    ureg = UnitRegistry()
+    um=re.search(r"\[(.+)\]",text).group(1) #ricerca unità di misura
+    u=ureg.Unit(um)
+    um_tex=f"{u:Lx}"
+    new_text=text.replace("["+um+"]","[]") #cancellazione unità di misure
+    new_text=_tex_escape(new_text) 
+    new_text=new_text.replace("[]",r"\(\left["+um_tex+r"\right]\)") #inserimento siunix dopo aver fatto l'escape dei caratteri speciali per latex
+    return new_text
+
+def _do_extra(to_do):
+    code_to_execute = compile("\n".join(to_do), "<string>", "exec")
+    exec(code_to_execute)
+
 def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_latex,
+    split_graphs=False,
+    titoli=["",""],
+    xlabel=None,
+    ylabel=None,
     path=".",
     xscale="linear",
     yscale="linear",
-    griglia_fit={"which" : "both", "ls" : "dashed", "color" : "gray"},
-    griglia_residui={"which" : "both", "ls" : "dashed", "color" : "gray"},
+    griglia_fit={
+        "which" : "both",
+        "ls" : "dashed",
+        "color" : "gray"
+    },
+    griglia_residui={
+        "which" : "both",
+        "ls" : "dashed",
+        "color" : "gray"
+    },
     style_fit={"fmt" : "o"},
     style_residui={"fmt" : "o"},
-    foglio="Foglio1"
+    foglio="Foglio1",
+    aggiunte=["a=1","print(a)","print(foglio)"]
 ):
     """
     funzione che genera:
 
-        - due pdf:
+        - un pdf:
 
-            - il grafico di fit
-        
-            - il grafico dei residui
+            - contenente il grafico di fit e quello dei residui
 
         - un file LaTeX:
         
@@ -75,6 +102,17 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         elenco dei nomi dei parametri ottimali che vengono ricavati dal fit
         verranno usati come contenuto della cella del titolo di una tabella latex contenente i valori numerici dei parametri
 
+    split_graphs: bool, optional
+        se True la funzione genera due file separati per il grafico dei residui e il grafico di fit (default: False)
+
+    titoli : list, optional
+        lista di due elementi con i due titoli per i due grafici, il primo è il grafico di fit e il secondo è il grafico dei 
+        residui (default: ["",""])
+    
+    xlabel, ylabel : str, optional
+        stringa da utilizzare per la generazione dei label del grafico, usare notezione LaTeX. Se ==None allora i nomi delle colonne
+        del file excel verranno utilizzati, convertendo automaticamente in LaTeX (default: None)
+
     path : str, optional
         path directory di partenza (default: ".")
 
@@ -90,6 +128,10 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     foglio : str, optional
         nome del foglio da cui prendere i dati (default: "Foglio1")
 
+    aggiunte : list, optional
+        lista di comandi da eseguire prima che il grafico venga salvato, usare ax1 per modificare il grafico di fit, ax1 per modificare
+        il grafico dei residui. nel caso di split_graphs=True le modifiche vengono apllicate al grafico dei residui (non usare ax1 e ax2 in questo caso)
+
     Examples
     ----------------------------
     >>> def retta(x,m,q):
@@ -153,6 +195,8 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     y=[e for e in excel[variabile_y].tolist() if e == e]
     dy=[e for e in excel[sigma_y].tolist() if e == e]
 
+    print(t,y,dy)
+
     #fit
     popt, pcov = curve_fit(f_fit,t,y,sigma=dy)
     
@@ -180,25 +224,83 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     #disegno grafico 
     pad=(max(t)-min(t))/20
     a = np.linspace(min(t)-pad if min(t)-pad>0 else 0, max(t)+pad, 1001)
-    plt.figure(file_result)
-    plt.xlabel(variabile_x)
-    plt.ylabel(variabile_y)
-    plt.xscale(xscale)
-    plt.yscale(yscale)
-    plt.grid(**griglia_fit)
-    plt.errorbar(t, y, dy, **style_fit)
-    plt.plot(a, f_fit(a, *popt)) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-    plt.savefig(path+'\\'+file_result+'.pdf')
 
-    #disegno grafico dei residui
-    plt.figure('Residui')
-    plt.xlabel(variabile_x)
-    plt.ylabel(variabile_y)
-    plt.grid(**griglia_residui)
-    y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-    plt.errorbar(t, y_res, dy, **style_residui)
-    plt.plot(a,a-a)
-    plt.savefig(path+'\\'+file_result+'_residui.pdf')
+    plt.rcParams.update({"text.usetex" : True, 
+        "font.family": "computer modern",
+        "text.latex.preamble": "\n".join([ # plots will use this preamble
+        r"\usepackage[utf8]{inputenc}",
+        r"\usepackage[T1]{fontenc}",
+        r"\usepackage{siunitx}",
+        ])})
+    
+    if split_graphs:
+        #grafico fit
+        plt.figure(file_result)
+        plt.title(titoli[0])
+        if xlabel != None:
+            plt.xlabel(xlabel)
+        else:
+            plt.xlabel(_converti_latex(variabile_x))
+        if ylabel != None:
+            plt.ylabel(ylabel)
+        else:
+            plt.ylabel(_converti_latex(variabile_y))
+        plt.xscale(xscale)
+        plt.yscale(yscale)
+        plt.grid(**griglia_fit)
+        plt.errorbar(t, y, dy, **style_fit)
+        plt.plot(a, f_fit(a, *popt)) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+        plt.savefig(path+'\\'+file_result+'.pdf')
+
+        #disegno grafico dei residui
+        plt.figure('Residui')
+        plt.title(titoli[1])
+        plt.xlabel(_converti_latex(variabile_x))
+        plt.ylabel(_converti_latex(variabile_y))
+        plt.grid(**griglia_residui)
+        y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+        plt.errorbar(t, y_res, dy, **style_residui)
+        plt.plot(a,a-a)
+        plt.savefig(path+'\\'+file_result+'_residui.pdf')
+    else:
+        #fit subplot
+        fig, (ax1, ax2) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [4, 1]}, sharex=True)
+        ax1.set_title(titoli[0])
+        ax1.set_xscale(xscale)
+        ax1.set_yscale(yscale)
+        ax1.grid(**griglia_fit)
+        ax1.errorbar(t, y, dy, **style_fit)
+        ax1.plot(a, f_fit(a, *popt)) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+        
+        #residui subplot
+        ax2.set_title(titoli[1])
+        ax2.margins(0, 0.2)
+        ax2.set_xscale(xscale)
+        ax2.set_yscale(yscale)
+        ax2.grid(**griglia_residui)
+        y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+        ax2.errorbar(t, y_res, dy, **style_residui)
+        ax2.plot(a, a-a)
+        ##adjust for x axi symmetry
+        low, high = ax2.get_ylim() 
+        bound = max(abs(low), abs(high)) # find the new limits
+        ax2.set_ylim(-bound, bound) # set new limits
+
+        #labels
+        if xlabel != None:
+            fig.supxlabel(xlabel, y=0.05)
+        else:
+            fig.supxlabel(_converti_latex(variabile_x), y=0.05)
+        if ylabel != None:
+            fig.supylabel(ylabel, x=0.03)
+        else:
+            fig.supylabel(_converti_latex(variabile_y), x=0.03)
+        
+        fig.tight_layout()
+
+        fig.savefig(path+'\\'+file_result+'.pdf')
+        fig.canvas.manager.set_window_title("Grafico") 
+        
     plt.show()
 
 def tabella(file_name,file_result,
@@ -261,19 +363,13 @@ def tabella(file_name,file_result,
         if result:
             print(result)
     """
-    ureg = UnitRegistry()
     #lettura file
     excel = pd.read_excel(path+'\\'+file_name, sheet_name=foglio, usecols=colonne)
     #scrittura file
     ##creazione header in stile LaTeX
     my_header=[]
     for col in excel.columns:
-        um=re.search(r"\[(.+)\]",col).group(1) #ricerca unità di misura
-        u=ureg.Unit(um)
-        um_tex=f"{u:Lx}"
-        new_col=col.replace("["+um+"]","[]") #cancellazione unità di misure
-        new_col=_tex_escape(new_col) 
-        new_col=new_col.replace("[]","\\(\\left["+um_tex+"\\right]\\)") #inserimento siunix dopo aver fatto l'escape dei caratteri speciali per latex
+        new_col=_converti_latex(col)
         my_header.append(new_col)
 
     latex = excel.to_latex(
